@@ -90,13 +90,20 @@ struct _FsRawSessionPrivate
   GstElement *capsfilter;
   GList *codecs;
   FsCodec *send_codec;
+  gboolean transmitter_linked;
+
+  GstElement *send_tee;
+  GstPad *send_tee_pad;
+
+  GstElement *transform_bin;
+  GstElement *fakesink;
+
+  GstElement *valve;
 
   FsTransmitter *transmitter;
 
   guint tos; /* Protected by conf lock */
 
-  gulong stream_direction_handler_id;
-
   GMutex *mutex; /* protects the conference */
 
 #ifdef DEBUG_MUTEXES
@@ -162,6 +169,9 @@ static GType
 fs_raw_session_get_stream_transmitter_type (FsSession *session,
     const gchar *transmitter);
 
+static GstElement *
+_create_transform_bin (FsRawSession *self, GError **error);
+
 static void
 fs_raw_session_class_init (FsRawSessionClass *klass)
 {
@@ -253,6 +263,10 @@ fs_raw_session_dispose (GObject *object)
   FsRawConference *conference = NULL;
   GstElement *valve = NULL;
   GstElement *capsfilter = NULL;
+  GstElement *transform = NULL;
+  GstElement *send_tee = NULL;
+  GstElement *fakesink = NULL;
+  GstPad *send_tee_pad = NULL;
   gulong handler_id = 0;
   FsTransmitter *transmitter = NULL;
   GstPad *media_sink_pad = NULL;
@@ -271,8 +285,8 @@ fs_raw_session_dispose (GObject *object)
     goto out;
 
   GST_OBJECT_LOCK (conference);
-  valve = self->valve;
-  self->valve = NULL;
+  valve = self->priv->valve;
+  self->priv->valve = NULL;
   GST_OBJECT_UNLOCK (conference);
 
   if (valve)
@@ -296,11 +310,6 @@ fs_raw_session_dispose (GObject *object)
     gst_object_unref (capsfilter);
   }
 
-  GST_OBJECT_LOCK (conference);
-  handler_id = self->priv->stream_direction_handler_id;
-  self->priv->stream_direction_handler_id = 0;
-  GST_OBJECT_UNLOCK (conference);
-
   if (self->priv->stream)
   {
     if (handler_id > 0 && self->priv->stream)
@@ -331,6 +340,52 @@ fs_raw_session_dispose (GObject *object)
     gst_object_unref (media_sink_pad);
   }
 
+  GST_OBJECT_LOCK (conference);
+  transform = self->priv->transform_bin;
+  self->priv->transform_bin = NULL;
+  GST_OBJECT_UNLOCK (conference);
+
+  if (transform)
+  {
+    gst_element_set_locked_state (transform, TRUE);
+    gst_bin_remove (conferencebin, transform);
+    gst_element_set_state (transform, GST_STATE_NULL);
+    gst_object_unref (transform);
+  }
+
+  GST_OBJECT_LOCK (conference);
+  fakesink = self->priv->fakesink;
+  self->priv->fakesink = NULL;
+  GST_OBJECT_UNLOCK (conference);
+
+  if (fakesink)
+  {
+    gst_element_set_locked_state (fakesink, TRUE);
+    gst_bin_remove (conferencebin, fakesink);
+    gst_element_set_state (fakesink, GST_STATE_NULL);
+    gst_object_unref (fakesink);
+  }
+
+  GST_OBJECT_LOCK (conference);
+  send_tee = self->priv->send_tee;
+  self->priv->send_tee = NULL;
+
+  send_tee_pad = self->priv->send_tee_pad;
+  self->priv->send_tee_pad = NULL;
+  GST_OBJECT_UNLOCK (conference);
+
+  if (send_tee)
+  {
+    gst_element_set_locked_state (send_tee, TRUE);
+    gst_bin_remove (conferencebin, send_tee);
+    gst_element_set_state (send_tee, GST_STATE_NULL);
+    gst_object_unref (send_tee);
+  }
+
+  if (send_tee_pad != NULL)
+    gst_object_unref (send_tee_pad);
+
+
   gst_object_unref (conference);
 
  out:
@@ -466,7 +521,6 @@ fs_raw_session_constructed (GObject *object)
 
   g_assert (self->priv->conference);
 
-
   tmp = g_strdup_printf ("send_capsfilter_%u", self->id);
   self->priv->capsfilter = gst_element_factory_make ("capsfilter", tmp);
   g_free (tmp);
@@ -486,6 +540,7 @@ fs_raw_session_constructed (GObject *object)
     self->priv->construction_error = g_error_new (FS_ERROR,
         FS_ERROR_CONSTRUCTION, "Could not add capsfilter to conference");
     gst_object_unref (self->priv->capsfilter);
+    self->priv->capsfilter = NULL;
     return;
   }
 
@@ -494,54 +549,185 @@ fs_raw_session_constructed (GObject *object)
     self->priv->construction_error = g_error_new (FS_ERROR,
         FS_ERROR_CONSTRUCTION,
         "Could not sync the send capsfilter's state with its parent");
-    gst_bin_remove (GST_BIN (self->priv->conference), self->priv->capsfilter);
     return;
   }
 
+  self->priv->transform_bin = _create_transform_bin (self, NULL);
 
-  tmp = g_strdup_printf ("send_valve_%u", self->id);
-  self->valve = gst_element_factory_make ("valve", tmp);
+  if (self->priv->transform_bin == NULL)
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION, "Could not create transform bin");
+    return;
+  }
+
+  gst_object_ref_sink (self->priv->transform_bin);
+  if (!gst_bin_add (GST_BIN (self->priv->conference),
+      self->priv->transform_bin))
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION, "Could not add transform bin to conference");
+    gst_object_unref (self->priv->transform_bin);
+    self->priv->transform_bin = NULL;
+    return;
+  }
+
+  if (!gst_element_link_pads (self->priv->transform_bin, "src",
+          self->priv->capsfilter, "sink"))
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION,
+        "Could not link send transformbin and capsfilter");
+    return;
+  }
+
+  if (!gst_element_sync_state_with_parent (self->priv->transform_bin))
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION,
+        "Could not sync the send transformbin's state with its parent");
+    return;
+  }
+
+  tmp = g_strdup_printf ("send_tee_%u", self->id);
+  self->priv->send_tee = gst_element_factory_make ("tee", tmp);
   g_free (tmp);
 
-  if (!self->valve)
+  if (!self->priv->send_tee)
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION, "Could not make send tee");
+    return;
+  }
+
+  gst_object_ref_sink (self->priv->send_tee);
+
+  if (!gst_bin_add (GST_BIN (self->priv->conference), self->priv->send_tee))
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION, "Could not add send tee to conference");
+    gst_object_unref (self->priv->send_tee);
+    self->priv->send_tee = NULL;
+    return;
+  }
+
+  self->priv->send_tee_pad = gst_element_get_request_pad (self->priv->send_tee,
+    "src%d");
+
+  if (self->priv->send_tee_pad == NULL)
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION,
+        "Could not create send tee pad");
+    return;
+  }
+
+  pad = gst_element_get_static_pad (self->priv->transform_bin, "sink");
+  if (pad == NULL)
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION,
+        "Could not get transformbin sink pad");
+    return;
+  }
+
+  if (GST_PAD_LINK_FAILED (gst_pad_link (self->priv->send_tee_pad, pad)))
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION,
+        "Could not link send tee and transformbin");
+    gst_object_unref (pad);
+    return;
+  }
+
+  gst_object_unref (pad);
+
+  self->priv->fakesink = gst_element_factory_make ("fakesink", NULL);
+  if (!self->priv->fakesink)
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION, "Could not make fakesink");
+    return;
+  }
+
+  gst_object_ref_sink (self->priv->fakesink);
+  if (!gst_bin_add (GST_BIN (self->priv->conference), self->priv->fakesink))
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION, "Could not add fakesink to conference");
+    gst_object_unref (self->priv->fakesink);
+    self->priv->fakesink = NULL;
+    return;
+  }
+
+  if (!gst_element_link_pads (self->priv->send_tee, "src%d",
+          self->priv->fakesink, "sink"))
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION,
+        "Could not link send send tee and fakesink");
+    return;
+  }
+
+  if (!gst_element_sync_state_with_parent (self->priv->fakesink))
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION,
+        "Could not sync the fakesinks state with its parent");
+    return;
+  }
+
+  if (!gst_element_sync_state_with_parent (self->priv->send_tee))
+  {
+    self->priv->construction_error = g_error_new (FS_ERROR,
+        FS_ERROR_CONSTRUCTION,
+        "Could not sync the send valve's state with its parent");
+    gst_bin_remove (GST_BIN (self->priv->conference), self->priv->send_tee);
+    return;
+  }
+
+  tmp = g_strdup_printf ("send_valve_%u", self->id);
+  self->priv->valve = gst_element_factory_make ("valve", tmp);
+  g_free (tmp);
+
+  if (!self->priv->valve)
   {
     self->priv->construction_error = g_error_new (FS_ERROR,
         FS_ERROR_CONSTRUCTION, "Could not make send valve");
     return;
   }
 
-  gst_object_ref_sink (self->valve);
+  gst_object_ref_sink (self->priv->valve);
 
-  if (!gst_bin_add (GST_BIN (self->priv->conference), self->valve))
+  if (!gst_bin_add (GST_BIN (self->priv->conference), self->priv->valve))
   {
     self->priv->construction_error = g_error_new (FS_ERROR,
         FS_ERROR_CONSTRUCTION, "Could not add valve to conference");
-    gst_object_unref (self->valve);
+    gst_object_unref (self->priv->valve);
+    self->priv->valve = NULL;
     return;
   }
 
-  g_object_set (G_OBJECT (self->valve), "drop", TRUE, NULL);
+  g_object_set (G_OBJECT (self->priv->valve), "drop", TRUE, NULL);
 
-  if (!gst_element_sync_state_with_parent (self->valve))
+  if (!gst_element_sync_state_with_parent (self->priv->valve))
   {
     self->priv->construction_error = g_error_new (FS_ERROR,
         FS_ERROR_CONSTRUCTION,
         "Could not sync the send valve's state with its parent");
-    gst_bin_remove (GST_BIN (self->priv->conference), self->valve);
     return;
   }
 
-  if (!gst_element_link_pads (self->valve, "src",
-          self->priv->capsfilter, "sink"))
+  if (!gst_element_link_pads (self->priv->valve, "src",
+          self->priv->send_tee, "sink"))
   {
     self->priv->construction_error = g_error_new (FS_ERROR,
         FS_ERROR_CONSTRUCTION,
-        "Could not link send valve and capsfilter");
+        "Could not link send valve and transformbin");
     return;
   }
 
-
-  pad = gst_element_get_static_pad (self->valve, "sink");
+  pad = gst_element_get_static_pad (self->priv->valve, "sink");
   tmp = g_strdup_printf ("sink_%u", self->id);
   self->priv->media_sink_pad = gst_ghost_pad_new (tmp, pad);
   g_free (tmp);
@@ -571,16 +757,43 @@ fs_raw_session_constructed (GObject *object)
     G_OBJECT_CLASS (fs_raw_session_parent_class)->constructed (object);
 }
 
+static GstElement *
+_create_transform_bin (FsRawSession *self, GError **error)
+{
+  FsMediaType mtype = self->priv->media_type;
+
+  if (mtype == FS_MEDIA_TYPE_AUDIO)
+    return gst_parse_bin_from_description_full (
+      "audioconvert ! audioresample ! audioconvert", TRUE, NULL,
+      GST_PARSE_FLAG_NONE, error);
+  else if (mtype == FS_MEDIA_TYPE_VIDEO)
+    return gst_parse_bin_from_description_full ("ffmpegcolorspace ! videoscale",
+        TRUE, NULL, GST_PARSE_FLAG_NONE, error);
+
+  g_set_error (error, FS_ERROR, FS_ERROR_NOT_IMPLEMENTED,
+    "No transform bin for this media type");
+  return NULL;
+}
+
 static void
 _stream_remote_codecs_changed (FsRawStream *stream, GParamSpec *pspec,
     FsRawSession *self)
 {
-  FsRawConference *conference = fs_raw_session_get_conference (self, NULL);
   GList *codecs;
   GstCaps *caps;
   FsCodec *codec = NULL;
+  GstPad *transform_pad;
+  GError *error = NULL;
+  GstElement *transform = NULL;
+  FsRawConference *conference = fs_raw_session_get_conference (self, &error);
+  gboolean changed;
+  FsStreamDirection direction;
 
-  g_object_get (stream, "remote-codecs", &codecs, NULL);
+  if (conference == NULL)
+    goto error;
+
+  g_object_get (stream, "remote-codecs", &codecs, "direction", &direction,
+      NULL);
 
   if (!codecs)
     return;
@@ -590,20 +803,71 @@ _stream_remote_codecs_changed (FsRawStream *stream, GParamSpec *pspec,
   else if (codecs && codecs->data)
     codec = codecs->data;
 
-  caps = fs_raw_codec_to_gst_caps (codec);
+  GST_OBJECT_LOCK (conference);
+  transform = self->priv->transform_bin;
+  self->priv->transform_bin = NULL;
+  GST_OBJECT_UNLOCK (conference);
 
+  /* Replace the transform bin */
+  if (transform != NULL)
+  {
+    gst_element_set_locked_state (transform, TRUE);
+    gst_element_set_state (transform, GST_STATE_NULL);
+    gst_bin_remove (GST_BIN (conference), transform);
+    g_object_unref (transform);
+  }
+
+  transform = _create_transform_bin (self, &error);
+
+  if (transform == NULL)
+    goto error;
+  gst_object_ref_sink (transform);
+
+  if (!gst_bin_add (GST_BIN (conference), transform))
+    goto error;
+
+  caps = fs_raw_codec_to_gst_caps (codec);
   if (self->priv->capsfilter)
     g_object_set (self->priv->capsfilter, "caps", caps, NULL);
+  gst_caps_unref (caps);
+
+  if (!gst_element_link_pads (transform, "src",
+          self->priv->capsfilter, "sink"))
+    goto error;
+
+  if (!gst_element_sync_state_with_parent (transform))
+    goto error;
+
+  transform_pad = gst_element_get_static_pad (transform, "sink");
+  if (transform_pad == NULL)
+    goto error;
+
+  if (GST_PAD_LINK_FAILED (gst_pad_link (self->priv->send_tee_pad,
+      transform_pad)))
+    goto error;
 
   GST_OBJECT_LOCK (conference);
-  if (!fs_codec_are_equal (self->priv->send_codec, codec))
+  self->priv->transform_bin = transform;
+  transform = NULL;
+
+  if (self->priv->codecs)
+    fs_codec_list_destroy (self->priv->codecs);
+  self->priv->codecs = codecs;
+
+  if ((changed = !fs_codec_are_equal (self->priv->send_codec, codec)))
   {
     if (self->priv->send_codec)
       fs_codec_destroy (self->priv->send_codec);
 
     self->priv->send_codec = fs_codec_copy (codec);
+  }
 
-    GST_OBJECT_UNLOCK (conference);
+  GST_OBJECT_UNLOCK (conference);
+
+  fs_raw_session_update_direction (self, direction);
+
+  if (changed)
+  {
     g_object_notify (G_OBJECT (self), "current-send-codec");
     gst_element_post_message (GST_ELEMENT (self->priv->conference),
         gst_message_new_element (GST_OBJECT (self->priv->conference),
@@ -612,18 +876,27 @@ _stream_remote_codecs_changed (FsRawStream *stream, GParamSpec *pspec,
                 "codec", FS_TYPE_CODEC, codec,
                 "secondary-codecs", FS_TYPE_CODEC_LIST, NULL,
                 NULL)));
-    GST_OBJECT_LOCK (conference);
   }
 
-  if (self->priv->codecs)
-    fs_codec_list_destroy (self->priv->codecs);
-  self->priv->codecs = fs_codec_list_copy (codecs);
-  GST_OBJECT_UNLOCK (conference);
-
   g_object_notify (G_OBJECT (self), "codecs");
 
-  gst_caps_unref (caps);
   gst_object_unref (conference);
+  return;
+
+error:
+  if (error != NULL)
+    fs_session_emit_error (FS_SESSION (self), error->code, error->message,
+          "Unable to change transform bin");
+  else
+    fs_session_emit_error (FS_SESSION (self), FS_ERROR_INTERNAL,
+          "Unable to change transform bin",
+          "Unknown error");
+
+  if (conference != NULL)
+    gst_object_unref (conference);
+
+  if (transform != NULL)
+    gst_object_unref (transform);
 }
 
 void
@@ -638,7 +911,7 @@ raw_session_remove_stream (FsRawSession *self,
   if (!conference)
     return;
 
-  g_object_set (G_OBJECT (self->valve), "drop", TRUE, NULL);
+  g_object_set (G_OBJECT (self->priv->valve), "drop", TRUE, NULL);
 
   GST_OBJECT_LOCK (conference);
   if (self->priv->stream == (FsRawStream *) stream)
@@ -658,9 +931,12 @@ raw_session_remove_stream (FsRawSession *self,
   gst_element_set_state (src, GST_STATE_NULL);
   gst_bin_remove (GST_BIN (conference), src);
 
-  gst_element_set_locked_state (sink, TRUE);
-  gst_element_set_state (sink, GST_STATE_NULL);
-  gst_bin_remove (GST_BIN (conference), sink);
+  if (gst_object_has_ancestor (GST_OBJECT (sink), GST_OBJECT (conference)))
+  {
+    gst_element_set_locked_state (sink, TRUE);
+    gst_element_set_state (sink, GST_STATE_NULL);
+    gst_bin_remove (GST_BIN (conference), sink);
+  }
 
   gst_object_unref (src);
   gst_object_unref (sink);
@@ -706,27 +982,26 @@ _add_transmitter_sink (FsRawSession *self,
     goto error;
   }
 
+  self->priv->transmitter_linked = TRUE;
+
   return TRUE;
 
 error:
   return FALSE;
 }
 
-static void
-_stream_direction_notify_cb (GObject *stream,
-                             GParamSpec *pspec,
-                             gpointer user_data)
+void
+fs_raw_session_update_direction (FsRawSession *self,
+  FsStreamDirection direction)
 {
-  FsRawSession *self = FS_RAW_SESSION (user_data);
-  FsStreamDirection direction;
+  /* Don't start sending before we have codecs */
+  if (!self->priv->codecs)
+    return;
 
-  g_object_get (stream, "direction", &direction, NULL);
-
-  if (direction & FS_DIRECTION_SEND)
+  if (direction & FS_DIRECTION_SEND && !self->priv->transmitter_linked)
   {
     GstElement *transmitter_sink;
     GError *error = NULL;
-    gulong handler_id = 0;
     FsRawConference *conference;
 
     conference = fs_raw_session_get_conference (self, &error);
@@ -750,15 +1025,13 @@ _stream_direction_notify_cb (GObject *stream,
       return;
     }
 
-    GST_OBJECT_LOCK (conference);
-    handler_id = self->priv->stream_direction_handler_id;
-    self->priv->stream_direction_handler_id = 0;
-    GST_OBJECT_UNLOCK (conference);
-
-    g_signal_handler_disconnect (stream, handler_id);
-
     gst_object_unref (conference);
   }
+
+  if (direction & FS_DIRECTION_SEND)
+    g_object_set (self->priv->valve, "drop", FALSE, NULL);
+  else
+    g_object_set (self->priv->valve, "drop", TRUE, NULL);
 }
 
 /**
@@ -877,20 +1150,6 @@ fs_raw_session_new_stream (FsSession *session,
       goto error;
     }
 
-    if (direction & FS_DIRECTION_SEND)
-    {
-      g_object_get (fstransmitter, "gst-sink", &transmitter_sink, NULL);
-
-      if (!_add_transmitter_sink (self, transmitter_sink, error))
-        goto error;
-    }
-    else
-    {
-      self->priv->stream_direction_handler_id =
-          g_signal_connect (new_stream, "notify::direction",
-          G_CALLBACK (_stream_direction_notify_cb), self);
-    }
-
     GST_OBJECT_LOCK (conference);
     if (self->priv->stream)
     {
