@@ -46,6 +46,7 @@
 #endif
 
 #include "fs-raw-stream.h"
+#include "fs-raw-session.h"
 
 #include <arpa/inet.h>
 #include <fcntl.h>
@@ -508,7 +509,6 @@ fs_raw_stream_set_property (GObject *object,
       if (g_value_get_flags (value) != self->priv->direction)
       {
         GstElement *recv_valve = NULL;
-        GstElement *session_valve = NULL;
         FsStreamTransmitter *st = NULL;
 
         if (!conference ||
@@ -521,33 +521,27 @@ fs_raw_stream_set_property (GObject *object,
 
         if (self->priv->recv_valve)
           recv_valve = gst_object_ref (self->priv->recv_valve);
-        if (self->priv->session->valve)
-          session_valve = gst_object_ref (self->priv->session->valve);
         if (self->priv->stream_transmitter)
           st = g_object_ref (self->priv->stream_transmitter);
 
         self->priv->direction = g_value_get_flags (value);
 
         GST_OBJECT_UNLOCK (conference);
+        fs_raw_session_update_direction (self->priv->session,
+          self->priv->direction);
+
         if (recv_valve)
           g_object_set (recv_valve, "drop",
               (self->priv->direction & FS_DIRECTION_RECV) ? FALSE : TRUE, NULL);
-        if (session_valve)
-          g_object_set (session_valve, "drop",
-              (self->priv->direction & FS_DIRECTION_SEND) ? FALSE : TRUE, NULL);
         if (st)
           g_object_set (st, "sending",
               (self->priv->direction & FS_DIRECTION_SEND) ? TRUE : FALSE, NULL);
         GST_OBJECT_LOCK (conference);
 
-        if (session_valve)
-          gst_object_unref (session_valve);
         if (recv_valve)
           gst_object_unref (recv_valve);
         if (st)
           g_object_unref (st);
-
-        g_object_notify (object, "direction");
       }
       break;
     case PROP_CONFERENCE:
@@ -822,9 +816,6 @@ fs_raw_stream_constructed (GObject *object)
   if (self->priv->recv_valve)
     g_object_set (self->priv->recv_valve, "drop",
         (self->priv->direction & FS_DIRECTION_RECV) ? FALSE : TRUE, NULL);
-  if (self->priv->session->valve)
-    g_object_set (self->priv->session->valve, "drop",
-        (self->priv->direction & FS_DIRECTION_SEND) ? FALSE : TRUE, NULL);
 
   if (G_OBJECT_CLASS (fs_raw_stream_parent_class)->constructed)
     G_OBJECT_CLASS (fs_raw_stream_parent_class)->constructed (object);
